require 'rubygems'
require_gem 'fxruby'

include Fox

class BezeditWindow < FXMainWindow
	attr_accessor :curves

	def initialize(app)
		super(app, "Bezedit", nil, nil, DECOR_ALL, 100, 100, 400, 300)

		@curves = []
		@selected_curve = nil
		@curve_label_index = 0
		@active_anchor = nil

		# Construct a horizontal frame to hold the main window's contents
		@contents = FXHorizontalFrame.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)
		@canvas = FXCanvas.new(@contents, nil, 0, LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT, 0, 0, 256,192);

		# Right pane for the buttons
		@buttonFrame = FXVerticalFrame.new(@contents, FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT, 0, 0, 0, 0, 10, 10, 10, 10)

		@new_curve_button = FXButton.new(@buttonFrame, "New curve")
		@delete_curve_button = FXButton.new(@buttonFrame, "Delete curve")
		@load_button = FXButton.new(@buttonFrame, "Load")
		@save_button = FXButton.new(@buttonFrame, "Save")
		@curve_list = FXList.new(@buttonFrame, nil, 0, LIST_SINGLESELECT | LAYOUT_FILL_X | LAYOUT_FILL_Y)

		@canvas.connect(SEL_PAINT) do |sender, sel, event|
			FXDCWindow.new(@canvas, event) do |dc|
				dc.foreground = @canvas.backColor
				dc.fillRectangle(event.rect.x, event.rect.y, event.rect.w, event.rect.h)
			end
			@curves.each_index do |i|
				@curves[i].draw(@canvas, @selected_curve == i)
			end
		end

		@canvas.connect(SEL_LEFTBUTTONPRESS) do |sender, sel, event|
			next if @selected_curve.nil?
			@active_anchor = @curves[@selected_curve].anchor_at(event.click_x, event.click_y)
			next if @active_anchor.nil?
			@anchor_offset_x = @active_anchor.x - event.click_x
			@anchor_offset_y = @active_anchor.y - event.click_y
			@active_anchor.draw(@canvas, true)
		end

		@canvas.connect(SEL_LEFTBUTTONRELEASE) do |sender, sel, event|
			next if @active_anchor.nil?
			@active_anchor.x = event.win_x + @anchor_offset_x
			@active_anchor.y = event.win_y + @anchor_offset_y
			@active_anchor = nil
			@canvas.update
		end

		@new_curve_button.connect(SEL_COMMAND) do |sender, selector, data|
			create_curve(10,10,100,10,100,100)
		end

		@curve_list.connect(SEL_SELECTED) do |sender, sel, i|
			@selected_curve = i
			@canvas.update
		end

		@delete_curve_button.connect(SEL_COMMAND) do |sender, sel, data|
			next if @selected_curve.nil?
			@curve_list.removeItem(@selected_curve)
			@curves.delete_at(@selected_curve)
			@selected_curve = nil
			@canvas.update
		end

		@save_button.connect(SEL_COMMAND) do |sender, sel, data|
			save_filename = FXFileDialog.getSaveFilename(self, "Save picture", "")
			next if save_filename == ""
			File.open(save_filename, "w") do |file|
				@curves.each do |curve|
					file.puts "\tdb #{curve.anchors[0].y}, #{curve.anchors[1].y}, #{curve.anchors[2].y}, #{curve.anchors[0].x}, #{curve.anchors[1].x}, #{curve.anchors[2].x}"
				end
			end
		end

		@load_button.connect(SEL_COMMAND) do |sender, sel, data|
			load_filename = FXFileDialog.getOpenFilename(self, "Load picture", "")
			next if load_filename == ""
			@curves.clear
			@curve_list.clearItems
			File.open(load_filename) do |file|
				file.each_line do |line|
					next unless line =~ /db\s+(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)/
					# in file, numbers are ay,by,cy,ax,bx,cx;
					# create_curve wants ax,ay,bx,by,cx,cy
					create_curve($4.to_i,$1.to_i,$5.to_i,$2.to_i,$6.to_i,$3.to_i)
				end
			end
		end

	end

	def create_curve(x0, y0, x1, y1, x2, y2)
		i = @curves.length
		lbl = @curve_label_index += 1
		@curves.push Curve.new(x0, y0, x1, y1, x2, y2)
		@curve_list.appendItem(FXListItem.new("curve #{lbl}"))
		@curve_list.selectItem(i, true) # notify to send a SEL_SELECTED event
	end
end

class CurveAnchor
	attr_accessor :x, :y

	def initialize(x, y)
		@x = x; @y = y;
	end

	def draw(canvas, active = false)
		FXDCWindow.new(canvas) do |dc|
			dc.foreground = active ? "red" : "green"
			dc.fillRectangle(@x, @y, 4, 4)
		end
	end

	def at?(x,y)
		(@x..(@x+7)) === x && (@y..(@y+7)) === y
	end
end

class Curve
	attr_accessor :anchors
	def initialize(x0, y0, x1, y1, x2, y2)
		@anchors = [CurveAnchor.new(x0,y0), CurveAnchor.new(x1, y1), CurveAnchor.new(x2, y2)]
	end

	def draw(canvas, selected = false)
		FXDCWindow.new(canvas) do |dc|
			dc.foreground = selected ? "blue" : "black"
			0.step(1, 0.01) do |i|
				xl1 = (1-i) * anchors[0].x + i * anchors[1].x
				yl1 = (1-i) * anchors[0].y + i * anchors[1].y
				xl2 = (1-i) * anchors[1].x + i * anchors[2].x
				yl2 = (1-i) * anchors[1].y + i * anchors[2].y
				x = (1-i) * xl1 + i * xl2
				y = (1-i) * yl1 + i * yl2
				dc.fillRectangle(x, y, 4, 2)
			end
		end
		if selected
			@anchors.each do |anchor|
				anchor.draw(canvas)
			end
		end
	end

	def anchor_at(x,y)
		@anchors.each { |anchor|
			return anchor if anchor.at?(x,y)
		}
		nil
	end
end

theApp = FXApp.new
mainWindow = BezeditWindow.new(theApp)
theApp.create
mainWindow.show
theApp.run
